"""Generate a poly(A) tail."""

from Bio import SeqIO
from Bio.SeqRecord import SeqRecord
from Bio.Seq import Seq
from random import choices
from typing import (List, Tuple)
import argparse


def generate_poly_a(
    length: int = 100,
    weights: Tuple[float, float, float, float] = (
        0.914, 0.028, 0.025, 0.033
    )
) -> str:
    """Generate a poly(A) tail of specified length and composition.

    This function generates a nucleotide sequence that has compositional
    statistics resembling those of poly(A) tails.

    Args:
        length: Length of the desired tail.
        weights: Tuple of relative `A`, `C`, `G` and `U` frequencies in
            the tail.

    Returns:
        The generated poly(A) tail.

    Raises:
        ValueError: The provided length is not a positive `int` or is
            too large (maximum length = 200).
        ValueError: One or more of the provided `weights` are not
            positive or all weights are zero.
    """
    max_len: int = 200
    bases: Tuple[str, str, str, str] = ('A', 'C', 'G', 'U')

    # check parameters
    if not isinstance(length, int):
        raise ValueError(
            f"The provided length is not an integer: {length}"
        )
    if not 1 <= int(length) <= max_len:
        raise ValueError(
            "The provided length is outside of the accepted range "
            f"(1-{max_len}): {length}"
        )
    if len(weights) != len(bases):
        raise ValueError(
            "There is not a weight provided for each of the bases '{bases}': "
            "{weights}"
        )
    try:
        sum(weights)
    except TypeError:
        raise ValueError(
            "At least one of the provided weights is not a number: {weights}"
        )
    if any(w < 0 for w in weights):
        raise ValueError(
            "At least one of the provided weights is negative: {weights}"
        )
    if all(w == 0 for w in weights):
        raise ValueError(f"All weights are zero: {weights}")

    # ensure that the values are normalized
    s: float = float(sum(weights))
    norm_weights: List[float] = [freq/s for freq in weights]
    tail_bases: List[str] = choices(bases, weights=norm_weights, k=length)
    return "".join(tail_bases)


def main():
    """Append poly A tails to fasta sequences and write to file.

    This function iterates over `input` fasta sequences from a specified file,
    appends tails of given `length` and composition specified by `weights`,
    and writes them to file specified as `output`.
    """
    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument('-i', '--input', required=True,
                        help='Fasta file to add tails to')
    parser.add_argument('-o', '--output', default="out.fa",
                        help='Name of the output Fasta file (default = out.fa)')
    parser.add_argument('--length', default=100, type=int,
                        help='Length of the desired tail (max value = 200)')
    parser.add_argument('--weights', default=(0.914, 0.028, 0.025, 0.033), type=float, nargs='+',
                        help="Tuple of relative A, C, G and U frequencies in the tail")
    args = parser.parse_args()

    new_records = []
    for fasta in SeqIO.parse(open(args.input), 'fasta'):
        new_seq = fasta.seq + Seq(generate_poly_a(args.length, args.weights))
        new_records.append(SeqRecord(new_seq, id=fasta.id, description=fasta.description))
    SeqIO.write(new_records, args.output, "fasta")


if __name__ == '__main__':
    main()
